@@ -570,6 +570,68 @@ G_DEFINE_BOXED_TYPE_WITH_CODE (ClutterPoint, clutter_point,
                                clutter_point_free,
                                CLUTTER_REGISTER_INTERVAL_PROGRESS (clutter_point_progress))
 
+static int
+clutter_point_compare_line (const ClutterPoint *p,
+                            const ClutterPoint *a,
+                            const ClutterPoint *b)
+{
+  float x1 = b->x - a->x;
+  float y1 = b->y - a->y;
+  float x2 = p->x - a->x;
+  float y2 = p->y - a->y;
+  float cross_z = x1 * y2 - y1 * x2;
+
+  if (cross_z > 0.f)
+    return 1;
+  else if (cross_z < 0.f)
+    return -1;
+  else
+    return 0;
+}
+
+/**
+ * clutter_point_inside_quadrilateral:
+ * @point: a #ClutterPoint to test
+ * @vertices: array of vertices of the quadrilateral, in clockwise order,
+ *            from top-left to bottom-left
+ *
+ * Determines whether a point is inside the convex quadrilateral provided,
+ * and not on any of its edges or vertices.
+ *
+ * Returns: %TRUE if @point is inside the quadrilateral
+ */
+gboolean
+clutter_point_inside_quadrilateral (const ClutterPoint *point,
+                                    const ClutterPoint *vertices)
+{
+  unsigned int i;
+  int first_side;
+
+  first_side = 0;
+
+  for (i = 0; i < 4; i++)
+    {
+      int side;
+
+      side = clutter_point_compare_line (point,
+                                         &vertices[i],
+                                         &vertices[(i + 1) % 4]);
+
+      if (side)
+        {
+          if (first_side == 0)
+            first_side = side;
+          else if (side != first_side)
+            return FALSE;
+        }
+    }
+
+  if (first_side == 0)
+    return FALSE;
+
+  return TRUE;
+}
+
 
 
 /*
@@ -1146,29 +1208,60 @@ clutter_rect_inset (ClutterRect *rect,
     rect->size.height = 0.f;
 }
 
+/**
+ * clutter_rect_scale:
+ * @rect: a #ClutterRect
+ * @s_x: an horizontal scale value
+ * @s_y: a vertical scale value
+ *
+ * Scale the rectangle coordinates and size by @s_x horizontally and
+ * @s_y vertically.
+ */
+void
+clutter_rect_scale (ClutterRect *rect,
+                    float        s_x,
+                    float        s_y)
+{
+  g_return_if_fail (rect != NULL);
+  g_return_if_fail (s_x > 0.f);
+  g_return_if_fail (s_y > 0.f);
+
+  clutter_rect_normalize_internal (rect);
+
+  rect->origin.x *= s_x;
+  rect->origin.y *= s_y;
+  rect->size.width *= s_x;
+  rect->size.height *= s_y;
+}
+
 /**
  * clutter_rect_clamp_to_pixel:
  * @rect: a #ClutterRect
  *
- * Rounds the origin of @rect downwards to the nearest integer, and rounds
- * the size of @rect upwards to the nearest integer, so that @rect is
- * updated to the smallest rectangle capable of fully containing the
- * original, fractional rectangle.
+ * Rounds the origin of @rect downwards to the nearest integer, and recompute the
+ * the size using the @rect origin and size rounded upwards to the nearest integer,
+ * so that @rect is updated to the smallest rectangle capable of fully containing
+ * the original, fractional rectangle in the coordinates space.
  *
  * Since: 1.12
  */
 void
 clutter_rect_clamp_to_pixel (ClutterRect *rect)
 {
+  float x2, y2;
+
   g_return_if_fail (rect != NULL);
 
   clutter_rect_normalize_internal (rect);
 
+  x2 = rect->origin.x + rect->size.width;
+  y2 = rect->origin.y + rect->size.height;
+
   rect->origin.x = floorf (rect->origin.x);
   rect->origin.y = floorf (rect->origin.y);
 
-  rect->size.width = ceilf (rect->size.width);
-  rect->size.height = ceilf (rect->size.height);
+  rect->size.width = ceilf (x2) - rect->origin.x;
+  rect->size.height = ceilf (y2) - rect->origin.y;
 }
 
 /**
