@@ -110,17 +110,16 @@ meta_kms_crtc_read_state (MetaKmsCrtc       *crtc,
                           MetaKmsImplDevice *impl_device,
                           drmModeCrtc       *drm_crtc)
 {
-  crtc->current_state = (MetaKmsCrtcState) {
-    .rect = {
-      .x = drm_crtc->x,
-      .y = drm_crtc->y,
-      .width = drm_crtc->width,
-      .height = drm_crtc->height,
-    },
-    .is_drm_mode_valid = drm_crtc->mode_valid,
-    .drm_mode = drm_crtc->mode,
+  crtc->current_state.rect = (MetaRectangle) {
+    .x = drm_crtc->x,
+    .y = drm_crtc->y,
+    .width = drm_crtc->width,
+    .height = drm_crtc->height,
   };
 
+  crtc->current_state.is_drm_mode_valid = drm_crtc->mode_valid;
+  crtc->current_state.drm_mode = drm_crtc->mode;
+
   read_gamma_state (crtc, impl_device, drm_crtc);
 }
 
@@ -133,6 +132,13 @@ meta_kms_crtc_update_state (MetaKmsCrtc *crtc)
   impl_device = meta_kms_device_get_impl_device (crtc->device);
   drm_crtc = drmModeGetCrtc (meta_kms_impl_device_get_fd (impl_device),
                              crtc->id);
+  if (!drm_crtc)
+    {
+      crtc->current_state.rect = (MetaRectangle) { };
+      crtc->current_state.is_drm_mode_valid = FALSE;
+      return;
+    }
+
   meta_kms_crtc_read_state (crtc, impl_device, drm_crtc);
   drmModeFreeCrtc (drm_crtc);
 }
