@@ -45,6 +45,8 @@
 #include "backends/meta-idle-monitor-private.h"
 #include "backends/meta-stage-private.h"
 #include "backends/x11/meta-clutter-backend-x11.h"
+#include "backends/x11/meta-event-x11.h"
+#include "backends/x11/meta-stage-x11.h"
 #include "backends/x11/meta-renderer-x11.h"
 #include "clutter/clutter.h"
 #include "clutter/x11/clutter-x11.h"
@@ -382,7 +384,7 @@ handle_host_xevent (MetaBackend *backend,
   if (!bypass_clutter)
     {
       handle_input_event (x11, event);
-      clutter_x11_handle_event (event);
+      meta_x11_handle_event (event);
     }
 
   XFreeEventData (priv->xdisplay, &event->xcookie);
@@ -788,14 +790,6 @@ meta_backend_x11_class_init (MetaBackendX11Class *klass)
 static void
 meta_backend_x11_init (MetaBackendX11 *x11)
 {
-  /* XInitThreads() is needed to use the "threaded swap wait" functionality
-   * in Cogl - see meta_renderer_x11_create_cogl_renderer(). We call it here
-   * to hopefully call it before any other use of XLib.
-   */
-  XInitThreads();
-
-  /* We do X11 event retrieval ourselves */
-  clutter_x11_disable_event_retrieval ();
 }
 
 Display *
@@ -810,7 +804,7 @@ Window
 meta_backend_x11_get_xwindow (MetaBackendX11 *x11)
 {
   ClutterActor *stage = meta_backend_get_stage (META_BACKEND (x11));
-  return clutter_x11_get_stage_window (CLUTTER_STAGE (stage));
+  return meta_x11_get_stage_window (CLUTTER_STAGE (stage));
 }
 
 void
